@@ -1,12 +1,15 @@
 import copy
 import json
+import logging
 import os
 import traceback
+from functools import cmp_to_key
 
 import requests
 from requests.exceptions import HTTPError
 
 from dkutils.constants import (
+    API_DELETE,
     API_GET,
     API_POST,
     API_PUT,
@@ -21,6 +24,7 @@ from dkutils.constants import (
     PARAMETERS,
     PARENT_KITCHEN,
     RECIPE,
+    SERVING_ERROR,
     STOPPED_STATUS_TYPES,
     VARIATION,
 )
@@ -29,6 +33,8 @@ from dkutils.validation import get_max_concurrency, skip_token_validation
 from dkutils.wait_loop import WaitLoop
 from .datetime_utils import get_utc_timestamp
 
+logger = logging.getLogger(__name__)
+
 # The servings API endpoint retrieves only 10 order runs by default. To retrieve them all, assume
 # 100K exceeds the max order runs a given order will ever contain.
 DEFAULT_SERVINGS_COUNT = 100000
@@ -130,6 +136,7 @@ class DataKitchenClient:
         self.kitchen = kitchen
         self.recipe = recipe
         self.variation = variation
+        self._valid_attributes = False
 
     @property
     def kitchen(self):
@@ -138,6 +145,7 @@ class DataKitchenClient:
     @kitchen.setter
     def kitchen(self, kitchen):
         self._kitchen = kitchen
+        self._valid_attributes = False
 
     def set_kitchen(self, kitchen):
         self.kitchen = kitchen
@@ -150,6 +158,7 @@ class DataKitchenClient:
     @recipe.setter
     def recipe(self, recipe):
         self._recipe = recipe
+        self._valid_attributes = False
 
     def set_recipe(self, recipe):
         self.recipe = recipe
@@ -162,6 +171,7 @@ class DataKitchenClient:
     @variation.setter
     def variation(self, variation):
         self._variation = variation
+        self._valid_attributes = False
 
     def set_variation(self, variation):
         self.variation = variation
@@ -171,12 +181,49 @@ class DataKitchenClient:
         """
         Ensure the properties required for the API request are all defined.
         """
+
+        def sort_args(a, b):
+            """
+            Sort so attributes are processed in Kitchen, Recipe, Variation order
+            """
+            if KITCHEN == a:
+                return -1
+            if RECIPE == a and VARIATION == b:
+                return -1
+            if a < b:
+                return -1
+            if a == b:
+                return 0
+            return 1
+
+        if self._valid_attributes:
+            return
         invalid_attributes = []
-        for attr_name in args:
+        attributes_to_check = set(args)
+        if VARIATION in attributes_to_check:
+            attributes_to_check.add(RECIPE)
+        if RECIPE in attributes_to_check:
+            attributes_to_check.add(KITCHEN)
+        for attr_name in sorted(attributes_to_check, key=cmp_to_key(sort_args)):
             if getattr(self, attr_name) is None:
                 invalid_attributes.append(attr_name)
+            elif attr_name == RECIPE and KITCHEN not in invalid_attributes:
+                recipes = self.get_recipes()
+                if self.recipe not in recipes:
+                    raise ValueError(
+                        f'{self.recipe} is not one of the available recipes: {",".join(recipes)}'
+                    )
+            elif attr_name == VARIATION and RECIPE not in invalid_attributes and KITCHEN not in invalid_attributes:
+                variations = self.get_variations()
+                if self.variation not in variations:
+                    raise ValueError(
+                        f'{self.variation} is not one of the available variations: {",".join(variations)}'
+                    )
+
         if invalid_attributes:
-            raise ValueError(f'Undefined attributes: ",".join({invalid_attributes})')
+            raise ValueError(f'Undefined attributes: {",".join(invalid_attributes)}')
+        else:
+            self._valid_attributes = True
 
     def _api_request(self, http_method, *args, is_json=True, **kwargs):
         """
@@ -206,7 +253,14 @@ class DataKitchenClient:
             response = api_request(api_path, headers=self._headers, json=kwargs)
         else:
             response = api_request(api_path, headers=self._headers, data=kwargs)
-        response.raise_for_status()
+
+        try:
+            response.raise_for_status()
+        except Exception:
+            if not skip_token_validation():
+                logger.error(f'Response Content:\n{response.content}')
+            raise
+
         return response
 
     def _validate_token(self):
@@ -281,6 +335,52 @@ class DataKitchenClient:
             parameters=parameters
         )
 
+    def delete_order(self, order_id):
+        """
+        Delete an Order. Kitchen attribute must be set prior to invoking this method.
+
+        Parameters
+        ----------
+        order_id : str
+            ID of the order being deleted.
+
+        Raises
+        ------
+        HTTPError
+            If the request fails.
+
+        Returns
+        ------
+        requests.Response
+            :class:`Response <Response>` object
+        """
+        self._ensure_attributes(KITCHEN)
+        return self._api_request(API_DELETE, 'order', 'delete', order_id, kitchen_name=self.kitchen)
+
+    def delete_order_run(self, order_run_id):
+        """
+        Delete an Order Run. Kitchen attribute must be set prior to invoking this method.
+
+        Parameters
+        ----------
+        order_run_id : str
+            ID of the order run being deleted.
+
+        Raises
+        ------
+        HTTPError
+            If the request fails.
+
+        Returns
+        ------
+        requests.Response
+            :class:`Response <Response>` object
+        """
+        self._ensure_attributes(KITCHEN)
+        return self._api_request(
+            API_DELETE, 'serving', 'delete', order_run_id, kitchen_name=self.kitchen
+        )
+
     def resume_order_run(self, order_run_id):
         """
         Resume a failed order run. Kitchen attribute must be set prior to invoking this method.
@@ -305,6 +405,205 @@ class DataKitchenClient:
             API_PUT, 'order', 'resume', order_run_id, kitchen_name=self.kitchen
         )
 
+    def get_orders(
+        self,
+        time_period_hours=None,
+        order_id_regex=None,
+        order_status=None,
+        order_run_status=None,
+        order_run_count=3
+    ):
+        """
+        Retrieve a dictionary of orders and their associated order runs based on the applied
+        filters. To filter based on recipe and variation, ensure these properties are set on this
+        client. Alternatively, set recipe and/or variation to None to remove filtering based on
+        these properties.
+
+        Parameters
+        ----------
+        time_period_hours : int, optional
+            Limit retrieved orders to those that started less than the provided number of hours ago.
+        order_id_regex : string, optional
+            Filter retrieved orders based on this provided order id regular expression
+        order_status : string, optional
+            Filter retrieved orders with this provided order status
+        order_run_status : string, optional
+            Filter retrieved order run with this provided order run status
+        order_run_count : int, optional
+            Limit the number of retrieved order runs (default is 3)
+
+        Raises
+        ------
+        HTTPError
+            If the request fails.
+        ValueError
+            If the kitchen attribute is not set
+
+        Returns
+        -------
+        orders : dict
+            Dictionary of orders and their associated order runs in the form::
+
+                {
+                    "customer": "DKImplementation",
+                    "kitchenname": "New_Customer_Training",
+                    "orders": [
+                        {
+                            "hid": "d1cea220-8c0a-11eb-8d5d-5acebe1b7ce3",
+                            "input_settings": {
+                                "customer_git_name": "im",
+                                "customer_git_org": "DKImplementation",
+                                "customer_name": "Implementation",
+                                "email": "user@gmail.com",
+                                "email_verified": true,
+                                "graph-setting": [
+                                    [
+                                        "Train_Model"
+                                    ]
+                                ],
+                                "login": "user@gmail.com",
+                                "parameters": {
+                                    "edges": [],
+                                    "nodes": [
+                                        "Train_Model"
+                                    ]
+                                },
+                                "role": "ADMIN",
+                                "schedule": "now",
+                                "user_id": "auth0|5d3b2624888ba60cf584c9d3"
+                            },
+                            "order-status": "ORDER_ERROR",
+                            "recipe": "Training_Sales_Forecast",
+                            "schedule": "now",
+                            "variation": "train_walmart_sales_forecasting_model"
+                        }
+                    ],
+                    "repo": "im",
+                    "servings": {
+                        "d1cea220-8c0a-11eb-8d5d-5acebe1b7ce3": {
+                            "servings": [
+                                {
+                                    "hid": "d5ccbef2-8c0a-11eb-bfad-1a3acc706153",
+                                    "order_id": "d1cea220-8c0a-11eb-8d5d-5acebe1b7ce3",
+                                    "orderrun_status": "Error in OrderRun",
+                                    "status": "SERVING_ERROR",
+                                    "timings": {
+                                        "duration": 120439,
+                                        "end-time": 1616526493384,
+                                        "start-time": 1616526372945
+                                    },
+                                    "variation_name": "train_walmart_sales_forecasting_model"
+                                }
+                            ],
+                            "total": 1
+                        }
+                    },
+                    "total-orders": 1
+                }
+        """
+        self._ensure_attributes(KITCHEN)
+        kwargs = {}
+        if self.recipe:
+            kwargs[RECIPE] = self.recipe
+        if self.variation:
+            kwargs[VARIATION] = self.variation
+        if time_period_hours:
+            kwargs['timePeriod'] = time_period_hours
+        if order_id_regex:
+            kwargs['search'] = order_id_regex
+        if order_status:
+            kwargs['orderStatus'] = order_status
+        if order_run_status:
+            kwargs['orderRunStatus'] = order_run_status
+        if order_run_count:
+            kwargs['servingsCount'] = order_run_count
+
+        return self._api_request(API_GET, 'order', 'status', self.kitchen, **kwargs).json()
+
+    def get_order_status(
+        self,
+        time_period_hours=None,
+        order_id_regex=None,
+        order_status=None,
+        order_run_status=None,
+        order_run_count=3
+    ):
+        """
+        Retrieve the order (and associated order runs) status details based on the applied
+        filters. To filter based on recipe and variation, ensure these properties are set on this
+        client. Alternatively, set recipe and/or variation to None to remove filtering based on
+        these properties.
+
+        Parameters
+        ----------
+        time_period_hours : int, optional
+            Limit retrieved orders to those that started less than the provided number of hours ago.
+        order_id_regex : string, optional
+            Filter retrieved orders based on this provided order id regular expression
+        order_status : string, optional
+            Filter retrieved orders with this provided order status
+        order_run_status : string, optional
+            Filter retrieved order run with this provided order run status
+        order_run_count : int, optional
+            Limit the number of retrieved order runs (default is 3)
+
+        Raises
+        ------
+        HTTPError
+            If the request fails.
+        ValueError
+            If the kitchen attribute is not set
+
+        Returns
+        -------
+        order_runs : list
+            A list of order runs sorted from most recent start-time to oldest in the form::
+
+                [
+                    {
+                        'hid': '66152846-e0ee-11ea-8280-12128c919b99',
+                        'order_id': 'a8f9ac78-e0ed-11ea-b3f4-56a20effdf97',
+                        'orderrun_status': 'OrderRun Completed',
+                        'status': 'COMPLETED_SERVING',
+                        'timings': {
+                            'duration': 22261,
+                            'end-time': 1597712533033,
+                            'start-time': 1597712510772
+                        }
+                    },
+                    {
+                        'hid': '3f57a7d0-e0ec-11ea-a65f-5edd64ee22f9',
+                        'order_id': '3942feda-e0ec-11ea-a00e-56a20effdf97',
+                        'orderrun_status': 'OrderRun Completed',
+                        'status': 'COMPLETED_SERVING',
+                        'timings': {
+                            'duration': 22676,
+                            'end-time': 1597711609459,
+                            'start-time': 1597711586783
+                        },
+                        'variation_name': 'variation1'
+                    }
+                ]
+
+        """
+        response_json = self.get_orders(
+            time_period_hours=time_period_hours,
+            order_id_regex=order_id_regex,
+            order_status=order_status,
+            order_run_status=order_run_status,
+            order_run_count=order_run_count
+        )
+
+        order_runs = []
+        if 'servings' in response_json and response_json['servings']:
+            for order in response_json['servings'].values():
+                order_runs.extend(order['servings'])
+
+        def sort_start_time(value):
+            return value['timings']['start-time']
+
+        return sorted(order_runs, key=sort_start_time, reverse=True)
+
     def get_order_runs(self, order_id):
         """
         Retrieve all the order runs associated with the provided order. The kitchen attribute
@@ -342,7 +641,7 @@ class DataKitchenClient:
             ).json()
             return api_response['servings']
         except HTTPError:
-            print(
+            logger.error(
                 f'No order runs found for provided order id ({order_id}) in kitchen {self.kitchen}'
             )
 
@@ -434,7 +733,7 @@ class DataKitchenClient:
         try:
             return self.get_order_run_details(order_run_id)['status']
         except HTTPError:
-            print(f'Order run retrieval failure:\n{traceback.format_exc()}')
+            logger.error(f'Order run retrieval failure:\n{traceback.format_exc()}')
             return None
 
     def monitor_order_run(self, sleep_secs, duration_secs, order_run_id):
@@ -501,7 +800,7 @@ class DataKitchenClient:
         return completed_order_runs
 
     def create_and_monitor_orders(
-        self, orders_details, sleep_secs, duration_secs, max_concurrent=None
+        self, orders_details, sleep_secs, duration_secs, max_concurrent=None, stop_on_error=False
     ):
         """
         Create the specified orders and wait for them to complete (or timeout after the specified
@@ -530,6 +829,9 @@ class DataKitchenClient:
         max_concurrent : integer or None
             Max number of orders to kick off concurrently. If None, all orders will be kicked off
             concurrently.
+        stop_on_error : boolean
+            If True, any order run failure will prevent new order runs from being created.
+            Otherwise, if False, all orders are submitted regardless of any failures.
 
         Returns
         -------
@@ -580,15 +882,16 @@ class DataKitchenClient:
         completed_orders = []
 
         wait_loop = WaitLoop(sleep_secs, duration_secs)
+        submit_new_orders = True
         while wait_loop:
             cur_completed_orders = []
             for active_order in active_orders:
                 self.kitchen = active_order[KITCHEN]
                 if active_order[ORDER_RUN_ID] is not None:
-                    active_order[ORDER_RUN_STATUS] = self.get_order_run_status(
-                        active_order[ORDER_RUN_ID]
-                    )
-                    if active_order[ORDER_RUN_STATUS] in STOPPED_STATUS_TYPES:
+                    order_status = self.get_order_run_status(active_order[ORDER_RUN_ID])
+                    active_order[ORDER_RUN_STATUS] = order_status
+                    submit_new_orders = not stop_on_error or order_status != SERVING_ERROR
+                    if order_status in STOPPED_STATUS_TYPES:
                         completed_orders.append(active_order)
                         cur_completed_orders.append(active_order)
                 else:
@@ -598,7 +901,7 @@ class DataKitchenClient:
 
             for completed_order in cur_completed_orders:
                 active_orders.remove(completed_order)
-                if len(queued_orders) > 0:
+                if len(queued_orders) > 0 and submit_new_orders:
                     active_orders.append(create_order(queued_orders.pop()))
 
             if len(active_orders) == 0:
@@ -607,7 +910,12 @@ class DataKitchenClient:
         return completed_orders, active_orders, queued_orders
 
     def resume_and_monitor_orders(
-        self, order_runs_details, sleep_secs, duration_secs, max_concurrent=None
+        self,
+        order_runs_details,
+        sleep_secs,
+        duration_secs,
+        max_concurrent=None,
+        stop_on_error=False
     ):
         """
         Resume the specified order runs and wait for them to complete (or timeout after the
@@ -630,6 +938,9 @@ class DataKitchenClient:
         max_concurrent : integer or None
             Max number of orders to kick off concurrently. If None, all orders will be kicked off
             concurrently.
+        stop_on_error : boolean
+            If True, any order run failure will prevent new order runs from being created.
+            Otherwise, if False, all orders are submitted regardless of any failures.
 
         Returns
         -------
@@ -677,15 +988,16 @@ class DataKitchenClient:
         completed_orders = []
 
         wait_loop = WaitLoop(sleep_secs, duration_secs)
+        submit_new_orders = True
         while wait_loop:
             cur_completed_orders = []
             for active_order in active_orders:
                 self.kitchen = active_order[KITCHEN]
                 if active_order[ORDER_RUN_ID] is not None:
-                    active_order[ORDER_RUN_STATUS] = self.get_order_run_status(
-                        active_order[ORDER_RUN_ID]
-                    )
-                    if active_order[ORDER_RUN_STATUS] in STOPPED_STATUS_TYPES:
+                    order_status = self.get_order_run_status(active_order[ORDER_RUN_ID])
+                    active_order[ORDER_RUN_STATUS] = order_status
+                    submit_new_orders = not stop_on_error or order_status != SERVING_ERROR
+                    if order_status in STOPPED_STATUS_TYPES:
                         completed_orders.append(active_order)
                         cur_completed_orders.append(active_order)
                 else:
@@ -698,7 +1010,7 @@ class DataKitchenClient:
 
             for completed_order in cur_completed_orders:
                 active_orders.remove(completed_order)
-                if len(queued_orders) > 0:
+                if len(queued_orders) > 0 and submit_new_orders:
                     active_orders.append(resume_order(queued_orders.pop()))
 
             if len(active_orders) == 0:
@@ -706,6 +1018,17 @@ class DataKitchenClient:
 
         return completed_orders, active_orders, queued_orders
 
+    def get_kitchens(self):
+        """
+        Retrieve a list of all kitchen names:
+
+        Returns
+        -------
+        list
+            List of all kitchen names.
+        """
+        return list(self._get_kitchens_info().keys())
+
     def update_kitchen_vault(
         self,
         prefix,
@@ -925,9 +1248,10 @@ class DataKitchenClient:
         payload = {"kitchen.json": kitchen_info}
         self._api_request(API_POST, 'kitchen', 'update', self.kitchen, **payload)
 
-    def get_overrides(self):
+    def get_overrides(self, override_names=None):
         """
-        Returns a dictionary containing the overrides for the current kitchen
+        Returns a dictionary containing the overrides for the current kitchen or if override_names is specified a
+        subset of overrides
 
         Raises
         ------
@@ -936,14 +1260,23 @@ class DataKitchenClient:
         ValueError
             If the kitchen attribute is not set
             If the name in the given kitchen_info does not match that of the current kitchen
+            If any of the names in override_names are not available in the current kitchen
 
         Returns
         -------
         dict
-            A dictionary containing the overides
+            A dictionary containing the overrides
 
         """
-        return self._get_kitchen_info()[RECIPE_OVERRIDES]
+        overrides = self._get_kitchen_info()[RECIPE_OVERRIDES]
+        if override_names:
+            missing = override_names - overrides.keys()
+            if missing:
+                raise ValueError(
+                    f'The following overrides are not available in the kitchen: {missing}'
+                )
+            overrides = {key: overrides[key] for key in override_names}
+        return overrides
 
     def update_overrides(self, overrides):
         """
@@ -994,6 +1327,62 @@ class DataKitchenClient:
         other_overrides = kitchens[other][RECIPE_OVERRIDES]
         return DictionaryComparator(my_overrides, other_overrides)
 
+    def get_override_names_that_do_not_exist(self, override_names):
+        """
+        Given a set of override names that could exist in the kitchen, return a set of the names of any that do not
+        exist in the kitchen.
+
+        Parameters
+        ----------
+            override_names : set
+                A set of override names that could exist in the kitchen
+
+        Raises
+        ------
+        HTTPError
+            If the request fails.
+        ValueError
+            If the kitchen attribute is not set
+            If the set of overrides is empty
+
+        Returns
+        -------
+        set
+            The set of any of given override names to do not exist in the kitchen. If all of the override names exist
+            then an empty set will be returned.
+        """
+        if not override_names:
+            raise ValueError("At least one override must be specified")
+        return override_names - self.get_overrides().keys()
+
+    def get_override_names_that_exist(self, override_names):
+        """
+        Given a set of override names that could exist in the kitchen, return a set of the names of that exist in the
+        kitchen.
+
+        Parameters
+        ----------
+            override_names : set
+                A set of override names that could exist in the kitchen
+
+        Raises
+        ------
+        HTTPError
+            If the request fails.
+        ValueError
+            If the kitchen attribute is not set
+            If the set of overrides is empty
+
+        Returns
+        -------
+        set
+            The set of any of given override names that exist in the kitchen. If none of the override names exist
+            then an empty set will be returned.
+        """
+        if not override_names:
+            raise ValueError("At least one override must be specified")
+        return override_names & self.get_overrides().keys()
+
     def get_kitchen_staff(self):
         """
         Returns a list containing the email addresses of the kitchen staff for the current kitchen
@@ -1046,7 +1435,7 @@ class DataKitchenClient:
 
         Parameters
         ----------
-        new_kitchen_staff:list
+        new_kitchen_staff : list
             A list containing the email addresses to be added to the kitchen staff
 
         Raises
@@ -1057,7 +1446,166 @@ class DataKitchenClient:
             If the kitchen attribute is not set
         """
         kitchen_staff = self.get_kitchen_staff()
+        if not kitchen_staff:
+            return
         for staff_member in new_kitchen_staff:
             if staff_member not in kitchen_staff:
                 kitchen_staff.append(staff_member)
         self.update_kitchen_staff(kitchen_staff)
+
+    def get_recipe(self, recipe_files=None, include_recipe_tree=False):
+        """
+        Retrieve the file structure and contents of a recipe. Using the default arguments, all
+        files and their contents will be returned. To return specific file contents, provide those
+        file paths in the recipe_files list. To return a subset of file contents, but the entire
+        recipe files hierarchy, set the include_recipe_tree to True.
+
+        Parameters
+        ----------
+        recipe_files : list or None
+            None, to return the contents of all files, or a list of file paths (relative to the
+            root of the recipe) to return contents of specific files.
+        include_recipe_tree : boolean
+            When set to True and used in conjunction with a non-empty list of recipe_files, a
+            recipe-tree key is added to the returned dictionary. This added field includes a
+            hierarchy of all files in a recipe and allows a user to return a subset of file
+            contents.
+
+        Raises
+        ------
+        HTTPError
+            If the request fails.
+        ValueError
+            If the kitchen or recipe attribute is not set, or if recipe_files is an empty list.
+
+        Returns
+        -------
+        dict
+            Dictionary of recipe files (for this example, recipe_files=['description.json'] and
+            include_recipe_tree==True) of the form::
+
+                {
+                    "ORIG_HEAD": "ef1d7cd1ed6e6371e9ea964b74e90f61de38fe68",
+                    "recipe-tree": {
+                        "Test": {
+                            "Test": [
+                                {
+                                    "filename": "description.json",
+                                    "json": "{    \"description\": \"\",     \"recipe-emails...
+                                    "sha": "20ceab58e77be39ccbfcd1cfae52fc6710eb47e5",
+                                    "type": "blob",
+                                    "url": "https://github.com/api/v3/repos/DataKitchen...
+                                },
+                                {
+                                    "filename": "variables.json",
+                                    "sha": "73b22809acd0c96f981f33bf9e7936780a5df397",
+                                    "type": "blob",
+                                    "url": "https://github.com/api/v3/repos/DataKitchen...
+                                },
+                                {
+                                    "filename": "variations.json",
+                                    "sha": "ba744dc4669fb44f1200cc86770772433c73ddf8",
+                                    "type": "blob",
+                                    "url": "https://github.com/api/v3/repos/DataKitchen...
+                                }
+                            ],
+                            "Test/resources": [
+                                {
+                                    "filename": "README.txt",
+                                    "sha": "6966c84b6474c42b28a51b3546ec27a5733abffa",
+                                    "type": "blob",
+                                    "url": "https://github.com/api/v3/repos/DataKitchen...
+                                }
+                            ]
+                        }
+                    },
+                    "recipes": {
+                        "Test": {
+                            "Test": [
+                                {
+                                    "filename": "description.json",
+                                    "json": "{    \"description\": \"\",     \"recipe-emails...
+                                    "sha": "20ceab58e77be39ccbfcd1cfae52fc6710eb47e5",
+                                    "type": "blob",
+                                    "url": "https://github.com/api/v3/repos/DataKitchen...
+                                }
+                            ]
+                        }
+                    }
+                }
+        """
+        self._ensure_attributes(KITCHEN, RECIPE)
+        if recipe_files or include_recipe_tree:
+            if isinstance(recipe_files, list) and len(recipe_files) == 0:
+                # API does not handle an empty recipe_files list graciously
+                raise ValueError('Argument recipe_files cannot be an empty array.')
+
+            kwargs = {'include-recipe-tree': include_recipe_tree, 'recipe-files': recipe_files}
+            return self._api_request(
+                API_POST, 'recipe', 'get', self.kitchen, self.recipe, **kwargs
+            ).json()
+        else:
+            return self._api_request(API_POST, 'recipe', 'get', self.kitchen, self.recipe).json()
+
+    def get_recipes(self):
+        """
+        Returns a list of recipe names.
+
+        Raises
+        ------
+        HTTPError
+            If the request fails.
+        ValueError
+            If the kitchen attribute is not set
+            If the current user is not in the kitchen_staff
+
+        Returns
+        -------
+        list
+            A list containing the recipe names.
+            For example::
+
+                [ 'Demo_Azure', 'Demo_NiFi' ]
+
+        """
+        self._ensure_attributes(KITCHEN)
+        kitchens = self._get_kitchens_info()
+        if self.kitchen not in kitchens:
+            raise ValueError(
+                f'{self.kitchen} is not one of the available kitchens: {",".join(kitchens.keys())}'
+            )
+        if kitchens[self.kitchen]['kitchen-staff'] and self._username not in kitchens[
+                self.kitchen]['kitchen-staff']:
+            raise ValueError(f'{self.kitchen} is not available to {self._username}')
+        return self._api_request(API_GET, 'kitchen', 'recipenames', self.kitchen).json()['recipes']
+
+    def get_variations(self):
+        """
+        Returns a dictionary that contains the variation-list from the variations.json file
+
+        Raises
+        ------
+        HTTPError
+            If the request fails.
+        ValueError
+            If the recipe attribute is not set
+
+        Returns
+        -------
+        dict
+            A dictionary containing the information from the variations.json file.
+            For example::
+
+                { "some_variation": {
+                    "description": "A descript of the variation",
+                    "graph-setting": "a_graph_setting"
+                    }
+                }
+
+        """
+        self._ensure_attributes(RECIPE)
+        response = self._api_request(
+            API_GET, 'recipe', 'file', self.kitchen, self.recipe, "variations.json"
+        )
+        contents = json.loads(response.json()['contents'])
+        return contents['variation-list']
